const args = process.argv.slice(2)
const mri = require('mri')
const path = require('path')
const puppeteer = require('puppeteer')

const OPTIONS = {
  args: [
    '--disable-gpu',
    '--disable-infobars',
    '--disable-setuid-sandbox',
    '--no-sandbox',
    '--window-size=1024,768',
  ],
  headless: false,
  ignoreHTTPSErrors: true,
}

class Sockpuppet {
  constructor() {
    this.argv = mri(args)
    this.options = OPTIONS
  }

  async start() {
    await this.preSetup()
    await this.setup()
    await this.postSetup()

    await this.run()

    await this.preTeardown()
    await this.teardown()
    await this.postTeardown()
  }

  async preSetup() {
  }

  async setup() {
    global.browser = await puppeteer.launch(this.options)
    await browser.pages().then(pages => global.page = pages[0])
    // FIXME: https://github.com/GoogleChrome/puppeteer/issues/1183
    // 694 = 768 - 74
    await page.setViewport({ width: 1024, height: 694 })
  }

  async postSetup() {
  }

  async run() {
    const specPath = this.getSpecPath()
    const spec = require(specPath)
    await spec()
  }

  getSpecPath() {
    const root = process.cwd()
    const spec = this.argv._[0] + '.js'

    return path.join(root, 'specs', spec)
  }

  async preTeardown() {
  }

  async teardown() {
    await browser.close()
  }

  async postTeardown() {
  }
}

module.exports = Sockpuppet
